<?php
namespace Zeedhi\PaymentPlatform\Implementations\Cielo;

use Zeedhi\PaymentPlatform\BankSlip\BankSlipPlatform;
use Zeedhi\PaymentPlatform\BankSlip\BankSlipTransaction;

use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyPlatform;
use Zeedhi\PaymentPlatform\InstantBuy\InstantBuyTransaction;

use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyPlatform;
use Zeedhi\PaymentPlatform\NonInstantBuy\NonInstantBuyTransaction;

use Zeedhi\PaymentPlatform\Consult\ConsultPlatform;
use Zeedhi\PaymentPlatform\Consult\Consult;

use Zeedhi\PaymentPlatform\Revoke\RevokePlatform;
use Zeedhi\PaymentPlatform\Revoke\Revoke;

use Zeedhi\PaymentPlatform\CreditCard;
use Zeedhi\PaymentPlatform\Customer;
use Zeedhi\PaymentPlatform\RequestURL;

use Zeedhi\PaymentPlatform\Exception;

class PaymentImpl implements InstantBuyPlatform, NonInstantBuyPlatform{
    const SANDBOX = 'SANDBOX';
    const ENDPOINT_SANDBOX = "https://apisandbox.cieloecommerce.cielo.com.br";

    const PRODUCTION = 'PRODUCTION';
    const ENDPOINT_PRODUCTION = "https://api.cieloecommerce.cielo.com.br/";

    const MAXREDIRS = 10;
    const TIMEOUT = 30;
    const POST = "POST";
    const PUT = "PUT";

    /** @var String */
    private $merchantKey;

    /** @var array */
    private $requestHeaders;

    /** @var HttpRequest */
    private $httpRequest;

    /** @var string */
    private $urlRequest;

    public function __construct($merchantKey, $merchantId, HttpRequest $httpRequest, $urlRequest) {

        $this->merchantKey = $merchantKey;
        $this->merchantId = $merchantId;
        $this->httpRequest = $httpRequest;
        $this->urlRequest = $urlRequest;
        $this->requestHeaders = array(
            'Content-Type:application/json',
            'Accept: application/json',
            'MerchantId: '.$this->merchantId,
            'Merchantkey: '.$this->merchantKey
        );

    }

    public function createToken(CreditCard $card, Customer $customer) {
        $request = array(
            "CustomerName" => $customer->getName(),
            "CardNumber" => $card->getNumbers(),
            "Holder" => $card->getHolderName(),
            "ExpirationDate" => $card->getExpDateMonth().'/'.$card->getExpDateYear(),
            "Brand" => $card->getFlag() === 'Mastercard' ? 'Master': $card->getFlag()
        );
        $responseData = $this->createRequest(RequestURL::CIELO_CARDS, self::POST, $request);
        return array(
            'InstantBuyKey' => $responseData['CardToken']
        );
    }
    
    public function instantBuyTransaction(InstantBuyTransaction $instantBuyTransaction) {
        $request = array(
            "MerchantOrderId" => $instantBuyTransaction->getTransactionId(),
            "Payment"         => array(
                "type"         => "CreditCard",
                "Amount"       => $instantBuyTransaction->getValue(),
                "Installments" => $instantBuyTransaction->getNrInstallment(),
                "CreditCard"   => array(
                    "CardToken" => $instantBuyTransaction->getCreditCardToken(),
                    "Brand" => $instantBuyTransaction->getFlag()
                )
            )
        );
        $responseData = $this->createRequest(RequestURL::CIELO_SALES, self::POST, $request);
        $captureUrl = $this->getCaptureUrl($responseData['Payment']);
        return array(
            'status' => $responseData['Payment']['ReturnCode'],
            'message' => $responseData['Payment']['ReturnMessage'],
            'captureUrl' => $captureUrl,
            'paymentId' => $responseData['Payment']['PaymentId'],
            'nsu' => isset($responseData['Payment']['ProofOfSale']) ? $responseData['Payment']['ProofOfSale'] : null
        );
    }

    public function nonInstantBuyTransaction(NonInstantBuyTransaction $nonInstantBuyTransaction) {
        $request = array(
            "MerchantOrderId" => $nonInstantBuyTransaction->getTransactionId(),
            "Payment"         => array(
                "type"         => "CreditCard",
                "Amount"       => $nonInstantBuyTransaction->getValue(),
                "Installments" => $nonInstantBuyTransaction->getNrInstallment(),
                "CreditCard"   => array(
                    "CardNumber"     => $nonInstantBuyTransaction->getCreditCard()->getNumbers(),
                    "Holder"         => $nonInstantBuyTransaction->getCreditCard()->getHolderName(),
                    "ExpirationDate" => $nonInstantBuyTransaction->getCreditCard()->getExpDateMonth().'/'.$nonInstantBuyTransaction->getCreditCard()->getExpDateYear(),
                    "SecurityCode"   => $nonInstantBuyTransaction->getCreditCard()->getCvv(),
                    "Brand"          => $nonInstantBuyTransaction->getCreditCard()->getFlag() === 'Mastercard' ? 'Master': $nonInstantBuyTransaction->getCreditCard()->getFlag()
                )
            )
        );
        $responseData = $this->createRequest(RequestURL::CIELO_SALES, self::POST, $request);
        $captureUrl = $this->getCaptureUrl($responseData['Payment']);
        return array(
            'status' => $responseData['Payment']['ReturnCode'],
            'message' => $responseData['Payment']['ReturnMessage'],
            'captureUrl' => $captureUrl,
            'paymentId' => $responseData['Payment']['PaymentId'],
            'nsu' => isset($responseData['Payment']['ProofOfSale']) ? $responseData['Payment']['ProofOfSale'] : null
        );
    }

    private function getCaptureUrl ($responseData) {
        $captureUrlIndex = array_search("capture", array_column($responseData['Links'], 'Rel'));
        $url = $responseData['Links'][$captureUrlIndex]['Href'];
        return substr($url, strpos($url, '/1/sales/'));
    }

    public function captureTransaction($url, $paymentId) {
        $request = array(
            'PaymentId' => $paymentId
        );
        $captureResponse = $this->createRequest($url, self::PUT, $request);
        return $captureResponse;
        
    }

    private function createRequest($url, $verb, $fields){
        $this->configCurl($url);
        //if($this->urlRequest == self::ENDPOINT_SANDBOX) {
            $this->httpRequest->setOption(CURLOPT_PROXY, "");
            $this->httpRequest->setOption(CURLOPT_PROXYUSERPWD, "");
            $this->httpRequest->setOption(CURLOPT_SSL_VERIFYPEER, false);
        //}
        $this->httpRequest->setOption(CURLOPT_CUSTOMREQUEST, $verb);
        $this->httpRequest->setOption(CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
        $this->httpRequest->setOption(CURLOPT_MAXREDIRS, self::MAXREDIRS);
        $this->httpRequest->setOption(CURLOPT_TIMEOUT, self::TIMEOUT);
        $this->httpRequest->setOption(CURLOPT_POSTFIELDS, json_encode($fields));
        return $this->responseTreatment();
    }

    private function responseTreatment() {
        $response = json_decode($this->httpRequest->execute(), true);
        $httpStatusCode = $this->httpRequest->getInfo(CURLINFO_HTTP_CODE);
        $this->httpRequest->close();
        if($httpStatusCode == '201' || $httpStatusCode == '200') {
            return $response;
        } else {
            throw Exception::apiCielo($httpStatusCode, $response[0]['Message']);
        }
    }

    private function configCurl($url) {
        $this->httpRequest->setUrl($this->urlRequest.$url);
        $this->httpRequest->setOption(CURLOPT_HTTPHEADER, $this->requestHeaders);
        $this->httpRequest->setOption(CURLOPT_RETURNTRANSFER, true);
    }

    public static function factory($merchantKey, $merchantId, $flag) {
        if($flag === self::SANDBOX){
            $urlRequest = self::ENDPOINT_SANDBOX;
        }else if($flag === self::PRODUCTION){
            $urlRequest = self::ENDPOINT_PRODUCTION;
        }
        return new self($merchantKey, $merchantId, new HttpRequest, $urlRequest);
    }
}