<?php
namespace Zeedhi\Framework\DB\Mongo;

class Mongo {

	/** @var \MongoDB */
	protected $db = null;
	/** @var \MongoClient */
	protected $mongo = null;

	protected $lazyConstructorParams = array();

	/**
	 * @param string $server
	 * @param string $port
	 * @param string $dbName Parameter optional
	 * @param array  $options
	 */
	public function __construct($server, $port, $dbName = null, $options = array()) {
		$this->lazyConstructorParams['server'] = $server;
		$this->lazyConstructorParams['port'] = $port;
		$this->lazyConstructorParams['dbName'] = $dbName;
		$this->lazyConstructorParams['options'] = $options;
	}

	protected function ensureInitialized() {
		if (!$this->mongo) {
			$this->lazyInit();
		}
	}

	/**
	 * @param $dbName
	 */
	public function setDbByName($dbName) {
		$this->ensureInitialized();
		$this->db = $this->mongo->$dbName;
	}

	protected function isDbSetted() {
		return $this->db !== null;
	}

	/**
	 * Find in database for documents that match with given criteria.
	 *
	 * @param string $collectionName
	 * @param array  $criteria
	 * @param array  $sort
	 * @param int    $limit
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return array
	 */
	public function find($collectionName, $criteria = array(), $sort = array(), $limit = 0) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		$collection = $this->getCollection($collectionName);
		$found = $collection->find($criteria)->limit($limit);
		$found->sort($sort);

		$result = array();
		while ($doc = $found->getNext()) {
			$result[] = $doc;
		}

		return $result;
	}

	/**
	 * @param string $collectionName
	 * @param array $criteria
	 *
	 * @return array
	 * @throws Exception
	 */
	public function aggregate($collectionName, array $criteria) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		$collection = $this->getCollection($collectionName);
		try {
			$result = $collection->aggregate($criteria);
			return $result['result'];
		} catch (\MongoException $e) {
			throw Exception::aggregateError($e);
		}
	}

	/**
	 * Insert the given register in database.
	 *
	 * @param string $collectionName
	 * @param mixed  $object
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return bool|array Returns an array containing the status of the
			insertion if the "w" option is set. Otherwise,
			returns TRUE if the inserted array is not empty (a MongoException
			will be thrown if the inserted array is empty).
	 */
	public function insert($collectionName, $object) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		return $this->getCollection($collectionName)->insert($object);
	}

	/**
	 * Insert the given registers in database (same as insertMany).
	 *
	 * @deprecated It's deprecated for insertMany. Reason: MongoDB from php5.x php7.x doens't include batchInsert into API.
	 * 
	 * @param string $collectionName
	 * @param array  $objects
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return bool|array Returns an array containing the status of the
			insertion if the "w" option is set. Otherwise, returns
			TRUE if the inserted array is not empty (a MongoException
			will be thrown if any inserted array is empty).
	 */
	public function batchInsert($collectionName, array $objects) {
		return $this->insertMany($collectionName, $objects);
	}

	/**
	 * Insert the given registers in database.
	 *
	 * @param string $collectionName
	 * @param array  $objects
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return bool|array Returns an array containing the status of the
			insertion if the "w" option is set. Otherwise,
			returns TRUE if the inserted array is not empty (a MongoException
			will be thrown if any
			inserted array is empty).
	 */
	public function insertMany($collectionName, array $objects) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		return $this->getCollection($collectionName)->batchInsert($objects);
	}

	/**
	 * Update a registers matched by criteria with alterations proposed by by update.
	 *
	 * @param string $collectionName
	 * @param array  $criteria
	 * @param mixed  $update
	 * @param bool   $upsert If true create a new document. Default to false.
	 * @param bool   $multi
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return boolean
	 */
	public function update($collectionName, $criteria, $update, $upsert = false, $multi = false) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		$options = array(
			"upsert" => $upsert,
			"multiple" => $multi,
		);

		return $this->getCollection($collectionName)->update($criteria, $update, $options);
	}

	/**
	 * Remove records that match with criteria in that collection.
	 *
	 * @param string $collectionName
	 * @param array  $criteria
	 * @param bool   $justOne If true delete only the first find record.
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return bool|array Returns an array containing the status of the removal
			if the "w" option is set. Otherwise, returns TRUE.
	 */
	public function remove($collectionName, $criteria, $justOne = true) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		$options = array(
			"justOne" => $justOne,
		);

		return $this->getCollection($collectionName)->remove($criteria, $options);
	}

	/**
	 * Drop the given collection.
	 *
	 * @param string $collectionName
	 *
	 * @throws Exception When db name is not setted.
	 *
	 * @return array Returns the database response.
	 */
	public function dropCollection($collectionName) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		return $this->getCollection($collectionName)->drop();
	}

	public function getCollectionNames() {
		$this->ensureInitialized();
		$this->validateDbSetted();
		return $this->db->getCollectionNames();
	}

	public function command(array $command) {
		$this->ensureInitialized();
		$this->validateDbSetted();
		return $this->db->command($command);
	}

	/**
	 * @param string $collectionName
	 *
	 * @return \MongoCollection
	 */
	public function getCollection($collectionName) {
		$this->ensureInitialized();
		return $this->db->$collectionName;
	}

	/**
	 * @return string
	 */
	protected function getMongoClassName() {
		$class = class_exists('\MongoClient') ? '\MongoClient' : '\Mongo';
		return $class;
	}

	protected function validateDbSetted() {
		if (!$this->isDbSetted()) {
			throw Exception::dbNotSetted();
		}
	}

	/**
	 * @param $server
	 * @param $port
	 * @param $dbName
	 * @param $options
	 */
	protected function init($server, $port, $dbName, $options) {
		$class = $this->getMongoClassName();

		if ($dbName) {
			$options['db'] = $dbName;
		}
		$this->mongo = new $class("mongodb://{$server}:{$port}", $options);

		if (!empty($options['db'])) {
			$this->setDbByName($dbName);
		}
	}

	protected function lazyInit() {
		$this->init(
			$this->lazyConstructorParams['server'],
			$this->lazyConstructorParams['port'],
			$this->lazyConstructorParams['dbName'],
			$this->lazyConstructorParams['options']
		);
	}

}